// This file is part of ptt2dtr.
//
// ptt2dtr is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// ptt2dtr is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ptt2dtr; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

#include "stdafx.h"
#include "ptt2dtr.h"
#include "ptt2dtrDlg.h"
#include "server.h"
#include ".\ptt2dtrdlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// Cptt2dtrDlg dialog



Cptt2dtrDlg::Cptt2dtrDlg(CWnd* pParent /*=NULL*/)
	: CDialog(Cptt2dtrDlg::IDD, pParent)
{
}

void Cptt2dtrDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_CHECK1, FiltEnChkBox);
	DDX_Control(pDX, IDC_CHECK4, ForwEnChkBox);
	DDX_Control(pDX, IDC_CHECK5, PTTButton);
	DDX_Control(pDX, IDC_COMBO1, ComboRealPort);
	DDX_Control(pDX, IDC_COMBO2, ComboVirtualPort);
	DDX_Control(pDX, IDC_COMBO3, ComboBaud);
	DDX_Control(pDX, IDC_EDIT2, EditBoxPort);
	DDX_Control(pDX, IDC_EDIT1, EditBoxPass);
	DDX_Control(pDX, IDC_CHECK6, ServEnabled);
	DDX_Control(pDX, IDC_COMBO4, ComboPTTPin);
	DDX_Control(pDX, IDC_CHECK2, ChkBoxSeparate);
	DDX_Control(pDX, IDC_COMBO5, ComboSeparate);
	DDX_Control(pDX, IDC_COMBO6, ComboRigType);
}

BEGIN_MESSAGE_MAP(Cptt2dtrDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDC_CHECK4, OnBnClickedCheck4)
	ON_MESSAGE( WU_EVENT_SERIALVIRT, OnVirtSerialMsg )
	ON_MESSAGE( WU_EVENT_SERIALREAL, OnRealSerialMsg )
	ON_BN_CLICKED(IDC_CHECK5, OnBnClickedCheck5)
	ON_MESSAGE( MYWM_NOTIFYICON, onTrayNotify )
	ON_WM_SIZE()
	ON_WM_CLOSE()
	ON_WM_SHOWWINDOW()
	ON_BN_CLICKED(IDC_BUTTON1, OnBnClickedButton1)
	ON_COMMAND( ID__SHOW, onSysTrayMenuShow )
	ON_COMMAND( ID__CLOSE, onSysTrayMenuClose )
	ON_BN_CLICKED(IDC_CHECK6, OnBnClickedCheck6)
	ON_BN_CLICKED(IDC_CHECK2, OnBnClickedCheck2)
	ON_CBN_SELCHANGE(IDC_COMBO5, OnCbnSelchangeCombo5)
	ON_CBN_SELCHANGE(IDC_COMBO6, &Cptt2dtrDlg::OnCbnSelchangeCombo6)
END_MESSAGE_MAP()

// Cptt2dtrDlg message handlers

void Cptt2dtrDlg::OnClose()
{
	PostMessage( WM_SIZE, SIZE_MINIMIZED, 0 );
}

void Cptt2dtrDlg::OnSize( UINT nType, int cx, int cy )
{
	if( nType == SIZE_MINIMIZED )
	{
		Shell_NotifyIcon( NIM_ADD, &m_hSysTrayIcon );
		ShowWindow( SW_HIDE );
	}
}

BOOL Cptt2dtrDlg::PreTranslateMessage( MSG* pMsg )
{
	switch( pMsg->message ) {
		case WM_KEYUP: // PTT push with space
		case WM_KEYDOWN:
			if( ForwEnChkBox.GetCheck() )
			{
				if( pMsg->wParam == VK_SPACE )
				{
					switchPTT( pMsg->message == WM_KEYUP ? 0 : 1 );
					PostMessage( WM_PAINT, 0, 0 );
					return 1;
				}
			}
			PostMessage( WM_PAINT, 0, 0 );
	}
	return 0;
}

// receiving from the virtual serial port
afx_msg LRESULT Cptt2dtrDlg::OnVirtSerialMsg( WPARAM wParam, LPARAM lParam )
{
	CSerial::EEvent eEvent = CSerial::EEvent(LOWORD(wParam));
    CSerial::EError eError = CSerial::EError(HIWORD(wParam));

	// lParam: User-defined 32-bit integer (type LPARAM)
    switch (eEvent)
    {
	    case CSerialMFC::EEventRecv:
			m_serialVirt.Read( m_szBuffer, sizeof( m_szBuffer ), &m_dwBytesRead );

			bool fMsgOk = true;
			if( FiltEnChkBox.GetCheck() )
			{
				if( m_sRigType == "Yaesu 857/897" )
				{
					// ptt on cmd
					if( m_szBuffer[0] == 0 && m_szBuffer[1] == 0 && m_szBuffer[2] == 0 && m_szBuffer[3] == 0 && m_szBuffer[4] == 0x08 )
					{
						switchPTT( 1 );
						fMsgOk = false;
					}

					// ptt off cmd
					if( m_szBuffer[0] == 0 && m_szBuffer[1] == 0 && m_szBuffer[2] == 0 && m_szBuffer[3] == 0 && (unsigned char)m_szBuffer[4] == 0x88 )
					{
						switchPTT( 0 );
						fMsgOk = false;
					}
				}
				if( m_sRigType == "Kenwood TS-570D" )
				{
					// ptt on cmd
					if( m_szBuffer[0] == 'T' && m_szBuffer[1] == 'X' && m_szBuffer[2] == ';' )
					{
						switchPTT( 1 );
						fMsgOk = false;
					}

					// ptt off cmd
					if( m_szBuffer[0] == 'R' && m_szBuffer[1] == 'X' && m_szBuffer[2] == ';' )
					{
						switchPTT( 0 );
						fMsgOk = false;
					}
				}
			}

			if( fMsgOk )
				m_serialReal.Write( (void*)m_szBuffer, m_dwBytesRead );

		break;
    }

	return 0;
}

// receiving from the real serial port
afx_msg LRESULT Cptt2dtrDlg::OnRealSerialMsg( WPARAM wParam, LPARAM lParam )
{
	CSerial::EEvent eEvent = CSerial::EEvent(LOWORD(wParam));
	CSerial::EError eError = CSerial::EError(HIWORD(wParam));

	// lParam: User-defined 32-bit integer (type LPARAM)
	switch (eEvent)
	{
	case CSerialMFC::EEventRecv:
		m_serialReal.Read( m_szBuffer, sizeof( m_szBuffer ), &m_dwBytesRead );

		m_serialVirt.Write( (void*)m_szBuffer, m_dwBytesRead );
		break;
	}

	return 0;
}

BOOL Cptt2dtrDlg::OnInitDialog()
{
	CDialog::OnInitDialog();
	m_hIcon = AfxGetApp()->LoadIcon(IDI_ICON1);

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	CString sTip(_T("ptt2dtr")); 

	m_hSysTrayIcon.cbSize = sizeof(NOTIFYICONDATA);
	m_hSysTrayIcon.hWnd = this->GetSafeHwnd();;
	m_hSysTrayIcon.uID = IDI_ICON1;
	m_hSysTrayIcon.uFlags = NIF_MESSAGE|NIF_ICON;
	m_hSysTrayIcon.uCallbackMessage = MYWM_NOTIFYICON;
	m_hSysTrayIcon.uFlags = NIF_MESSAGE|NIF_ICON|NIF_TIP; 
	m_hSysTrayIcon.hIcon = LoadIcon(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDI_ICON1));
	lstrcpyn( m_hSysTrayIcon.szTip, (LPCTSTR)sTip, sizeof(m_hSysTrayIcon.szTip) );

	m_trayMenu.LoadMenu( IDR_MENU1 );
	Shell_NotifyIcon( NIM_ADD, &m_hSysTrayIcon );

	m_iniFile.SetPath( "ptt2dtr.ini" );
	m_iniFile.ReadFile();
	m_sRealPort = m_iniFile.GetValue( "ptt2dtr", "realport", "COM1" ).c_str();
	m_sSeparate = m_iniFile.GetValue( "ptt2dtr", "separatepttport", "COM1" ).c_str();
	m_sVirtPort = m_iniFile.GetValue( "ptt2dtr", "virtport", "COM2" ).c_str();
	m_sBaudRate = m_iniFile.GetValue( "ptt2dtr", "baudrate", "9600" ).c_str();
	m_nPTTPin = m_iniFile.GetValueI( "ptt2dtr", "pttpin", 0 );
	m_sRigType = m_iniFile.GetValue( "ptt2dtr", "rigtype", "Yaesu 857/897" ).c_str();

	if( CSerial::CheckPort( "COM1" ) == CSerial::EPortAvailable )
		ComboRealPort.AddString( "COM1" );
	if( CSerial::CheckPort( "COM2" ) == CSerial::EPortAvailable )
		ComboRealPort.AddString( "COM2" );
	if( CSerial::CheckPort( "COM3" ) == CSerial::EPortAvailable )
		ComboRealPort.AddString( "COM3" );
	if( CSerial::CheckPort( "COM4" ) == CSerial::EPortAvailable )
		ComboRealPort.AddString( "COM4" );
	if( CSerial::CheckPort( "COM5" ) == CSerial::EPortAvailable )
		ComboRealPort.AddString( "COM5" );
	if( CSerial::CheckPort( "COM6" ) == CSerial::EPortAvailable )
		ComboRealPort.AddString( "COM6" );
	if( CSerial::CheckPort( "COM7" ) == CSerial::EPortAvailable )
		ComboRealPort.AddString( "COM7" );
	if( CSerial::CheckPort( "COM8" ) == CSerial::EPortAvailable )
		ComboRealPort.AddString( "COM8" );
	if( CSerial::CheckPort( "COM9" ) == CSerial::EPortAvailable )
		ComboRealPort.AddString( "COM9" );
	if( CSerial::CheckPort( "COM10" ) == CSerial::EPortAvailable )
		ComboRealPort.AddString( "COM10" );
	if( CSerial::CheckPort( "\\\\.\\CNCA0" ) == CSerial::EPortAvailable )
		ComboRealPort.AddString( "CNCA0" );
	if( CSerial::CheckPort( "\\\\.\\CNCB0" ) == CSerial::EPortAvailable )
		ComboRealPort.AddString( "CNCB0" );
	if( CSerial::CheckPort( "\\\\.\\CNCA1" ) == CSerial::EPortAvailable )
		ComboRealPort.AddString( "CNCA1" );
	if( CSerial::CheckPort( "\\\\.\\CNCB1" ) == CSerial::EPortAvailable )
		ComboRealPort.AddString( "CNCB1" );
	ComboRealPort.SetCurSel( ComboRealPort.FindString( 0, m_sRealPort ) );

	if( CSerial::CheckPort( "COM1" ) == CSerial::EPortAvailable )
		ComboSeparate.AddString( "COM1" );
	if( CSerial::CheckPort( "COM2" ) == CSerial::EPortAvailable )
		ComboSeparate.AddString( "COM2" );
	if( CSerial::CheckPort( "COM3" ) == CSerial::EPortAvailable )
		ComboSeparate.AddString( "COM3" );
	if( CSerial::CheckPort( "COM4" ) == CSerial::EPortAvailable )
		ComboSeparate.AddString( "COM4" );
	if( CSerial::CheckPort( "COM5" ) == CSerial::EPortAvailable )
		ComboSeparate.AddString( "COM5" );
	if( CSerial::CheckPort( "COM6" ) == CSerial::EPortAvailable )
		ComboSeparate.AddString( "COM6" );
	if( CSerial::CheckPort( "COM7" ) == CSerial::EPortAvailable )
		ComboSeparate.AddString( "COM7" );
	if( CSerial::CheckPort( "COM8" ) == CSerial::EPortAvailable )
		ComboSeparate.AddString( "COM8" );
	if( CSerial::CheckPort( "COM9" ) == CSerial::EPortAvailable )
		ComboSeparate.AddString( "COM9" );
	if( CSerial::CheckPort( "COM10" ) == CSerial::EPortAvailable )
		ComboSeparate.AddString( "COM10" );
	if( CSerial::CheckPort( "\\\\.\\CNCA0" ) == CSerial::EPortAvailable )
		ComboSeparate.AddString( "CNCA0" );
	if( CSerial::CheckPort( "\\\\.\\CNCB0" ) == CSerial::EPortAvailable )
		ComboSeparate.AddString( "CNCB0" );
	if( CSerial::CheckPort( "\\\\.\\CNCA1" ) == CSerial::EPortAvailable )
		ComboSeparate.AddString( "CNCA1" );
	if( CSerial::CheckPort( "\\\\.\\CNCB1" ) == CSerial::EPortAvailable )
		ComboSeparate.AddString( "CNCB1" );
	ComboSeparate.SetCurSel( ComboSeparate.FindString( 0, m_sSeparate ) );
	if( m_iniFile.GetValueI( "ptt2dtr", "useseparatepttport", 0 ) )
		ChkBoxSeparate.SetCheck( true );

	if( CSerial::CheckPort( "\\\\.\\CNCA0" ) == CSerial::EPortAvailable )
		ComboVirtualPort.AddString( "CNCA0" );
	if( CSerial::CheckPort( "\\\\.\\CNCB0" ) == CSerial::EPortAvailable )
		ComboVirtualPort.AddString( "CNCB0" );
	if( CSerial::CheckPort( "\\\\.\\CNCA1" ) == CSerial::EPortAvailable )
		ComboVirtualPort.AddString( "CNCA1" );
	if( CSerial::CheckPort( "\\\\.\\CNCB1" ) == CSerial::EPortAvailable )
		ComboVirtualPort.AddString( "CNCB1" );
	if( CSerial::CheckPort( "COM1" ) == CSerial::EPortAvailable )
		ComboVirtualPort.AddString( "COM1" );
	if( CSerial::CheckPort( "COM2" ) == CSerial::EPortAvailable )
		ComboVirtualPort.AddString( "COM2" );
	if( CSerial::CheckPort( "COM3" ) == CSerial::EPortAvailable )
		ComboVirtualPort.AddString( "COM3" );
	if( CSerial::CheckPort( "COM4" ) == CSerial::EPortAvailable )
		ComboVirtualPort.AddString( "COM4" );
	if( CSerial::CheckPort( "COM5" ) == CSerial::EPortAvailable )
		ComboVirtualPort.AddString( "COM5" );
	if( CSerial::CheckPort( "COM6" ) == CSerial::EPortAvailable )
		ComboVirtualPort.AddString( "COM6" );
	if( CSerial::CheckPort( "COM7" ) == CSerial::EPortAvailable )
		ComboVirtualPort.AddString( "COM7" );
	if( CSerial::CheckPort( "COM8" ) == CSerial::EPortAvailable )
		ComboVirtualPort.AddString( "COM8" );
	if( CSerial::CheckPort( "COM9" ) == CSerial::EPortAvailable )
		ComboVirtualPort.AddString( "COM9" );
	if( CSerial::CheckPort( "COM10" ) == CSerial::EPortAvailable )
		ComboVirtualPort.AddString( "COM01" );
	ComboVirtualPort.SetCurSel( ComboVirtualPort.FindString( 0, m_sVirtPort ) );

	ComboBaud.AddString( "1200" );
	ComboBaud.AddString( "2400" );
	ComboBaud.AddString( "4800" );
	ComboBaud.AddString( "9600" );
	ComboBaud.AddString( "14400" );
	ComboBaud.AddString( "19200" );
	ComboBaud.AddString( "38400" );
	ComboBaud.AddString( "57600" );
	ComboBaud.AddString( "115200" );
	ComboBaud.SetCurSel( ComboBaud.FindString( 0, m_sBaudRate ) );

	ComboPTTPin.AddString( "DTR" );
	ComboPTTPin.AddString( "RTS" );
	ComboPTTPin.AddString( "DTR+RTS" );
	ComboPTTPin.SetCurSel( m_nPTTPin );

	ComboRigType.AddString( "Yaesu 857/897" );
	ComboRigType.AddString( "Kenwood TS-570D" );
	ComboRigType.SetCurSel( ComboRigType.FindString( 0, m_sRigType ) );

	toggleForwardEnable( m_iniFile.GetValueI( "ptt2dtr", "forwardenableatstartup", 0 ) );
	FiltEnChkBox.SetCheck( m_iniFile.GetValueI( "ptt2dtr", "filterenableatstartup", 1 ) );

	m_hServerThread = INVALID_HANDLE_VALUE;

	ServEnabled.SetCheck( m_iniFile.GetValueI( "remoteptt", "enabled", 0 ) );
	EditBoxPass.SetWindowText( m_iniFile.GetValue( "remoteptt", "pass", "ptt2dtr" ).c_str() );
	EditBoxPort.SetWindowText( m_iniFile.GetValue( "remoteptt", "port", "5454" ).c_str() );
	toggleServEnable( m_iniFile.GetValueI( "remoteptt", "enabled", 0 ) );

	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void Cptt2dtrDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
// the minimized window.
HCURSOR Cptt2dtrDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

int Cptt2dtrDlg::ShowError( LONG lError, LPCTSTR lptszMessage )
{
	// Generate a message text
	TCHAR tszMessage[256];
	wsprintf(tszMessage,_T("%s\n(error code %d)"), lptszMessage, lError);

	// Display message-box and return with an error-code
	::MessageBox( 0, tszMessage, "Error", MB_ICONERROR|MB_OK );
	return 1;
}

HANDLE g_hEndServerThread;
int servPort = 0;
CString servPass;
void Cptt2dtrDlg::toggleServEnable( int fState )
{
	if( fState )
	{
		EditBoxPass.GetWindowText( servPass );
		CString sPort;
		EditBoxPort.GetWindowText( sPort );
		servPort = atoi( (LPCTSTR)sPort );
		if( servPort == 0 || sPort.GetLength() == 0 )
		{
			EditBoxPort.EnableWindow( true );
			EditBoxPass.EnableWindow( true );
			ServEnabled.SetCheck( false );
			return;
		}

		g_hEndServerThread = CreateEvent( 0, 0, 0, NULL );
		m_hServerThread = AfxBeginThread( serverThread, 0 );

		EditBoxPort.EnableWindow( false );
		EditBoxPass.EnableWindow( false );
	}
	else
	{
		if( m_hServerThread != INVALID_HANDLE_VALUE )
		{
			SetEvent( g_hEndServerThread );
			while( WaitForSingleObject( g_hEndServerThread, 0 ) != WAIT_OBJECT_0 )
				Sleep(1);
			m_hServerThread = INVALID_HANDLE_VALUE;
		}
		EditBoxPort.EnableWindow( true );
		EditBoxPass.EnableWindow( true );
	}
}

void Cptt2dtrDlg::toggleForwardEnable( int fState )
{
	if( fState )
	{
		m_serialVirt.Close();
		ComboVirtualPort.GetLBText( ComboVirtualPort.GetCurSel(), m_sVirtPort );
		ComboBaud.GetLBText( ComboBaud.GetCurSel(), m_sBaudRate );

		CSerial::EBaudrate baudRate;
		if( m_sBaudRate == "1200" )
			baudRate = CSerial::EBaud1200;
		if( m_sBaudRate == "2400" )
			baudRate = CSerial::EBaud2400;
		if( m_sBaudRate == "4800" )
			baudRate = CSerial::EBaud4800;
		if( m_sBaudRate == "9600" )
			baudRate = CSerial::EBaud9600;
		if( m_sBaudRate == "19200" )
			baudRate = CSerial::EBaud19200;
		if( m_sBaudRate == "38400" )
			baudRate = CSerial::EBaud38400;
		if( m_sBaudRate == "57600" )
			baudRate = CSerial::EBaud57600;
		if( m_sBaudRate == "115200" )
			baudRate = CSerial::EBaud115200;

		LONG lLastError = m_serialVirt.Open( "\\\\.\\" + m_sVirtPort, this->m_hWnd, WU_EVENT_SERIALVIRT );
		if ( lLastError != ERROR_SUCCESS )
		{
			ShowError( m_serialVirt.GetLastError(), _T("Unable to open virtual COM-port") );
			ForwEnChkBox.SetCheck( 0 );
			FiltEnChkBox.EnableWindow( false );
			return;
		}
		lLastError = m_serialVirt.Setup( baudRate, CSerial::EData8, CSerial::EParNone, CSerial::EStop1 );
		if (lLastError != ERROR_SUCCESS)
		{
			ShowError( m_serialVirt.GetLastError(), _T("Unable to set COM-port setting of virtual serial port") );
			ForwEnChkBox.SetCheck( 0 );
			FiltEnChkBox.EnableWindow( false );
			return;
		}
		lLastError = m_serialVirt.SetupHandshaking( CSerial::EHandshakeOff );
		if (lLastError != ERROR_SUCCESS)
		{
			ShowError( m_serialVirt.GetLastError(), _T("Unable to set COM-port handshaking of virtual serial port") );
			ForwEnChkBox.SetCheck( 0 );
			FiltEnChkBox.EnableWindow( false );
			return;
		}

		m_serialReal.Close();
		ComboRealPort.GetLBText( ComboRealPort.GetCurSel(), m_sRealPort );
		lLastError = m_serialReal.Open( "\\\\.\\" + m_sRealPort, this->m_hWnd, WU_EVENT_SERIALREAL );
		if ( lLastError != ERROR_SUCCESS )
		{
			ShowError( m_serialReal.GetLastError(), _T("Unable to open real COM-port") );
			ForwEnChkBox.SetCheck( 0 );
			FiltEnChkBox.EnableWindow( false );
			return;
		}
		lLastError = m_serialReal.Setup( baudRate, CSerial::EData8, CSerial::EParNone, CSerial::EStop1 );
		if (lLastError != ERROR_SUCCESS)
		{
			ShowError( m_serialReal.GetLastError(), _T("Unable to set COM-port setting of real serial port") );
			ForwEnChkBox.SetCheck( 0 );
			FiltEnChkBox.EnableWindow( false );
			return;
		}
		lLastError = m_serialReal.SetupHandshaking( CSerial::EHandshakeOff );
		if (lLastError != ERROR_SUCCESS)
		{
			ShowError( m_serialReal.GetLastError(), _T("Unable to set COM-port handshaking of real serial port") );
			ForwEnChkBox.SetCheck( 0 );
			FiltEnChkBox.EnableWindow( false );
			return;
		}

		if( ChkBoxSeparate.GetCheck() )
		{
			m_serialSeparate.Close();
			lLastError = m_serialSeparate.Open( "\\\\.\\" + m_sSeparate, this->m_hWnd, WU_EVENT_SERIALSEPARATE );
			if ( lLastError != ERROR_SUCCESS )
			{
				ShowError( m_serialSeparate.GetLastError(), _T("Unable to open separate COM-port") );
				ForwEnChkBox.SetCheck( 0 );
				FiltEnChkBox.EnableWindow( false );
				return;
			}
			lLastError = m_serialSeparate.Setup( baudRate, CSerial::EData8, CSerial::EParNone, CSerial::EStop1 );
			if (lLastError != ERROR_SUCCESS)
			{
				ShowError( m_serialSeparate.GetLastError(), _T("Unable to set COM-port setting of separate serial port") );
				ForwEnChkBox.SetCheck( 0 );
				FiltEnChkBox.EnableWindow( false );
				return;
			}
			lLastError = m_serialSeparate.SetupHandshaking( CSerial::EHandshakeOff );
			if (lLastError != ERROR_SUCCESS)
			{
				ShowError( m_serialSeparate.GetLastError(), _T("Unable to set COM-port handshaking of separate serial port") );
				ForwEnChkBox.SetCheck( 0 );
				FiltEnChkBox.EnableWindow( false );
				return;
			}
		}

		m_nPTTPin = ComboPTTPin.GetCurSel();

		ForwEnChkBox.SetCheck( true );
		FiltEnChkBox.EnableWindow( true );
		PTTButton.EnableWindow( true );
		PTTButton.SetFocus();
		ComboRealPort.EnableWindow( false );
		ComboVirtualPort.EnableWindow( false );
		ComboBaud.EnableWindow( false );
		ComboPTTPin.EnableWindow( false );
		ComboSeparate.EnableWindow( false );
		ComboRigType.EnableWindow( false );
		ChkBoxSeparate.EnableWindow( false );
	}
	else
	{
		m_serialVirt.Close();
		m_serialReal.Close();
		m_serialSeparate.Close();
		FiltEnChkBox.EnableWindow( false );
		PTTButton.EnableWindow( false );
		ForwEnChkBox.SetCheck( false );
		ComboRealPort.EnableWindow( true );
		ComboVirtualPort.EnableWindow( true );
		ComboBaud.EnableWindow( true );
		ComboPTTPin.EnableWindow( true );
		ChkBoxSeparate.EnableWindow( true );
		ComboRigType.EnableWindow( true );
		if( ChkBoxSeparate.GetCheck() )
			ComboSeparate.EnableWindow( true );
		else
			ComboSeparate.EnableWindow( false );
	}
}

void Cptt2dtrDlg::OnBnClickedCheck4()
{
	toggleForwardEnable( ForwEnChkBox.GetCheck() );
}

void Cptt2dtrDlg::switchPTT( int fState )
{
	if( fState )
	{
		PTTButton.SetCheck( false );

		if( !ChkBoxSeparate.GetCheck() )
		{
			switch( m_nPTTPin )
			{
				case 0:
					m_serialReal.SetDTR();
					break;
				case 1:
					m_serialReal.SetRTS();
					break;
				case 2:
					m_serialReal.SetRTS();
					m_serialReal.SetDTR();
			}
		}
		else
		{
			switch( m_nPTTPin )
			{
			case 0:
				m_serialSeparate.SetDTR();
				break;
			case 1:
				m_serialSeparate.SetRTS();
				break;
			case 2:
				m_serialSeparate.SetRTS();
				m_serialSeparate.SetDTR();
			}
		}

		if( ::GetLastError() == ERROR_SUCCESS )
		{
			PTTButton.SetCheck( true );
		}
	}
	else
	{
		PTTButton.SetCheck( false );
		m_serialReal.ClrDTR();
		m_serialReal.ClrRTS();
		m_serialSeparate.ClrDTR();
		m_serialSeparate.ClrRTS();
	}
}

void Cptt2dtrDlg::OnBnClickedCheck5()
{
	switchPTT( PTTButton.GetCheck() );
}

LRESULT Cptt2dtrDlg::onTrayNotify( WPARAM wParam, LPARAM lParam )
{
	if( lParam == WM_LBUTTONDBLCLK )
	{
		Shell_NotifyIcon( NIM_DELETE, &m_hSysTrayIcon );
		ShowWindow( SW_NORMAL );
		SetForegroundWindow();
		SetFocus();
	}

	if( lParam == WM_RBUTTONUP )
	{
		CPoint pt;
		GetCursorPos( &pt );
		m_trayMenu.GetSubMenu(0)->TrackPopupMenu( TPM_RIGHTALIGN|TPM_LEFTBUTTON|TPM_RIGHTBUTTON, pt.x, pt.y, this );
	} 

	return TRUE;
}

afx_msg void Cptt2dtrDlg::OnShowWindow( BOOL bShow, UINT nStatus )
{
	ShowWindow( SW_HIDE );
}

void Cptt2dtrDlg::OnBnClickedButton1()
{
	MessageBox(
		"ptt2dtr translates CAT PTT commands into COM port DTR line\n" \
		"triggering by hooking between Ham Radio Deluxe and the COM\n" \
		"port used to control the radio.\n\n" \
		"My Yaesu FT-857D transmits the signal coming from the MIC input\n" \
		"instead of the line in when PTT is toggled via CAT command.\n\n" \
		"To use ptt2dtr:\n\n" \
		"  - first download and install com0com from com0com.sourceforge.net,\n" \
		"  - using com0com's GUI setup, rename CNCA0 to for example COM2 and\n" \
		"CNCB0 to for example COM3 (if you don't change these port names to\n" \
		"COMx format, HRD won't see them)\n" \
		"  - in ptt2dtr, select your real and virtual COM port\n" \
		"(for example COM1 and COM2), your rig's type, then enable forwarding\n" \
		"(and CAT PTT filtering).\n" \
		"  - fire up HRD and set your radio's COM port to the other\n" \
		"virtual port (for example COM3). Now you have COM port PTT.\n\n" \
		"You can enable remote PTT triggering via TCP/IP server, use\n" \
		"simpleptt.exe as the client.\n\n"
		"Hint: use space bar for easy PTT toggling\n\n" \
		"ptt2dtr by Nonoo\nMore info: http://www.nonoo.hu/"
		, "Help", MB_ICONINFORMATION );
}

afx_msg void Cptt2dtrDlg::onSysTrayMenuShow()
{
	onTrayNotify( 0, WM_LBUTTONDBLCLK );
}

afx_msg void Cptt2dtrDlg::onSysTrayMenuClose()
{
	// saving settings
	m_iniFile.SetValueI( "ptt2dtr", "forwardenableatstartup", ForwEnChkBox.GetCheck() );
	m_iniFile.SetValueI( "ptt2dtr", "filterenableatstartup", FiltEnChkBox.GetCheck() );
	m_iniFile.SetValue( "ptt2dtr", "realport", (LPCTSTR)m_sRealPort );
	m_iniFile.SetValue( "ptt2dtr", "virtport", (LPCTSTR)m_sVirtPort );
	m_iniFile.SetValueI( "ptt2dtr", "useseparatepttport", ChkBoxSeparate.GetCheck() );
	m_iniFile.SetValue( "ptt2dtr", "separatepttport", (LPCTSTR)m_sSeparate );
	m_iniFile.SetValue( "ptt2dtr", "baudrate", (LPCTSTR)m_sBaudRate );
	m_iniFile.SetValueI( "ptt2dtr", "pttpin", m_nPTTPin );
	m_iniFile.SetValue( "ptt2dtr", "rigtype", (LPCTSTR)m_sRigType );
	m_iniFile.SetValueI( "remoteptt", "enabled", ServEnabled.GetCheck() );
	CString tmp;
	EditBoxPass.GetWindowText( tmp );
	m_iniFile.SetValue( "remoteptt", "pass", (LPCTSTR)tmp );
	EditBoxPort.GetWindowText( tmp );
	m_iniFile.SetValue( "remoteptt", "port", (LPCTSTR)tmp );
	m_iniFile.WriteFile();

	Shell_NotifyIcon( NIM_DELETE, &m_hSysTrayIcon );
	DestroyWindow();
}

void Cptt2dtrDlg::OnBnClickedCheck6()
{
	toggleServEnable( ServEnabled.GetCheck() );
}

void Cptt2dtrDlg::OnBnClickedCheck2()
{
	ComboSeparate.EnableWindow( ChkBoxSeparate.GetCheck() );
}

void Cptt2dtrDlg::OnCbnSelchangeCombo5()
{
	ComboSeparate.GetLBText( ComboSeparate.GetCurSel(), m_sSeparate );
}

void Cptt2dtrDlg::OnCbnSelchangeCombo6()
{
	ComboRigType.GetLBText( ComboRigType.GetCurSel(), m_sRigType );
}
