// This file is part of ptt2dtr.
//
// ptt2dtr is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// ptt2dtr is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ptt2dtr; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

#include "stdafx.h"
#include "ptt2dtr.h"
#include "ptt2dtrDlg.h"

extern Cptt2dtrDlg dlg;
extern HANDLE g_hEndServerThread;
extern CString servPass;
extern int servPort;

typedef struct {
	SOCKET clientSocket;
	bool authenticated;
	char inBuf[100];
	int inPos;
} t_client;

int readSckLine( t_client& tClient )
{
	char in;
	int res;
	
	res = recv( tClient.clientSocket, &in, 1, 0 );
	if( res > 0 )
	{
		tClient.inBuf[tClient.inPos++] = in;
	}
	
	if( res == 0 ) // res == 0 -> client disconnected
		return -1;
	if( in == 10 || tClient.inPos == sizeof(tClient.inBuf) - 1 )
	{
		tClient.inBuf[tClient.inPos] = 0;
		tClient.inPos = 0;
		return 1;
	}

	return 0;
}

UINT serverThread( LPVOID pParam )
{
	SOCKET serverSocket;
	WSADATA wsaData;

	sockaddr_in local;
	int wsaret = WSAStartup( 0x101, &wsaData );
	if( wsaret != 0 )
	{
		dlg.ShowError( wsaret, "WinSock init error" );
		return 0;
	}

	local.sin_family = AF_INET;
	local.sin_addr.s_addr = INADDR_ANY;
	local.sin_port = htons((u_short)servPort);

	serverSocket = socket( AF_INET, SOCK_STREAM, 0 );

	if( serverSocket == INVALID_SOCKET )
	{
		dlg.ShowError( servPort, "Can't open the given port!" );
		return 0;
	}

	if( bind( serverSocket,(sockaddr*)&local, sizeof(local) ) != 0 )
		return 0;

	if( listen( serverSocket, 10 ) != 0 )
		return 0;

	t_client newClient;
	CArray<t_client> clients;
	sockaddr_in from;
	int fromlen = sizeof(from);
	char temp2[50];
	int res;
	int i;

	u_long nonBlock = 1;
	ioctlsocket( serverSocket, FIONBIO, &nonBlock );

	newClient.authenticated = false;
	newClient.clientSocket = INVALID_SOCKET;
	newClient.inPos = 0;

	while( WaitForSingleObject( g_hEndServerThread, 0 ) != WAIT_OBJECT_0 )
	{
		Sleep( 1 );
		newClient.clientSocket = accept( serverSocket, (struct sockaddr*)&from, &fromlen );

		if( newClient.clientSocket != INVALID_SOCKET )
		{
			// new client connected
			clients.Add( newClient );
			newClient.clientSocket = INVALID_SOCKET;
		}

		for( i = 0; i < clients.GetSize(); i++ )
		{
			res = readSckLine( clients[i] );
			if( res == -1 )
			{
				// error
				closesocket( clients[i].clientSocket );
				clients.RemoveAt( i );
				continue;
			}

			if( res == 0 )
				continue;

			if( !clients[i].authenticated )
			{
				sprintf_s( temp2, "PASS %s", (LPCTSTR)servPass );

				if( strncmp( clients[i].inBuf, temp2, servPass.GetLength() + 5 ) )
				{
					// passwords does not match
					send( clients[i].clientSocket, "err\r\n", 5, 0 );
					closesocket( clients[i].clientSocket );
					clients.RemoveAt( i );
					continue;
				}
				send( clients[i].clientSocket, "ok\r\n", 4, 0 );
				clients[i].authenticated = true;
				continue;
			}

			if( strncmp( clients[i].inBuf, "P0", 2 ) == 0 )
			{
				dlg.switchPTT( false );
				send( clients[i].clientSocket, "P0\r\n", 4, 0 );
				continue;
			}
			if( strncmp( clients[i].inBuf, "P1", 2 ) == 0 )
			{
				dlg.switchPTT( true );
				send( clients[i].clientSocket, "P1\r\n", 4, 0 );
				continue;
			}
			send( clients[i].clientSocket, "err\r\n", 5, 0 );
		}
	}

	for( i = 0; i < clients.GetSize(); i++ )
	{
		closesocket( clients[i].clientSocket );
	}

	closesocket( serverSocket );
	WSACleanup();
	SAFE_CLOSE_HANDLE( g_hEndServerThread );
	return 0;
}
